"use strict";
/**
 * @author Riccardo La Marca
 *
 * @brief Problems-releated Commands:
 *  - list   [List and fetch a number of problems from leetcode]
 *  - fetch  [Fetch a single problems from leetcode]
 *  - detail [Provides more informations about a fetched problem]
 *  - create [Creates a local instance of a fetched problem]
 *  - daily  [Fetch the current daily question]
 */
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || (function () {
    var ownKeys = function(o) {
        ownKeys = Object.getOwnPropertyNames || function (o) {
            var ar = [];
            for (var k in o) if (Object.prototype.hasOwnProperty.call(o, k)) ar[ar.length] = k;
            return ar;
        };
        return ownKeys(o);
    };
    return function (mod) {
        if (mod && mod.__esModule) return mod;
        var result = {};
        if (mod != null) for (var k = ownKeys(mod), i = 0; i < k.length; i++) if (k[i] !== "default") __createBinding(result, mod, k[i]);
        __setModuleDefault(result, mod);
        return result;
    };
})();
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.daily_command = exports.create_command = exports.detail_command = exports.fetch_command = exports.list_command = void 0;
const pprint_1 = require("../pprint");
const printer_1 = require("../utils/printer");
const chalk_1 = __importDefault(require("chalk"));
const lc = __importStar(require("../leetcode"));
const generic = __importStar(require("../utils/general"));
const formatter = __importStar(require("../utils/formatter"));
const ListCommand = async (_, state) => {
    const spinner = new pprint_1.Spinner("Fetching Problem List with provided filters");
    spinner.start();
    const category = state.variables["CATEGORY"].value; // The category filter
    const limit = state.variables["LIMIT"].value; // The limit filter
    const skip = state.variables["SKIP"].value; // The skip filter
    const difficulty = state.variables["DIFFICULTY"].value.toUpperCase();
    var diff_filter = difficulty;
    if (difficulty === "ALL")
        diff_filter = undefined;
    // We need to set the cookies for obtain the current status
    let header = formatter.FormatCookies(state.cookies);
    const problems_data = await lc.FetchProblemList({
        categorySlug: category,
        limit: limit,
        skip: skip,
        filters: { difficulty: diff_filter }
    }, header);
    spinner.stop();
    if (!problems_data)
        return state;
    spinner.changeMessage("Fetching each problem details");
    spinner.start();
    // Then for each problem fetched, gathers other details
    const fetched_problems = await Promise.all(problems_data.problemsetQuestionList.questions.map(async (value) => {
        const title = value.titleSlug;
        const question_data = await lc.FetchQuestion({ titleSlug: title });
        if (!question_data) {
            console.warn(chalk_1.default.yellowBright("[WARNING] Unable to fetch more details " +
                `for problem titled: ${title}`));
            return null;
        }
        const merged_result = generic.MergeStructures(value, question_data.question);
        merged_result.link = `https://leetcode.com/problems/${title}/description`;
        return merged_result;
    }));
    spinner.stop();
    const final_result = fetched_problems.filter((value) => value !== null);
    if (final_result.length === 0)
        return state;
    state.fetchedProblems = { count: final_result.length, questions: final_result };
    (0, printer_1.PrintUsedFilters)(state.variables);
    await (0, printer_1.PrintProblemsSummary)(state.fetchedProblems, state.variables);
    return state;
};
// List command - Lists a number of problems
exports.list_command = {
    group: 'Problems',
    name: 'List Problems Command',
    command: 'list',
    syntax: /^list$/,
    callback: ListCommand,
    help: 'list - Fetches and Lists a number of problems from leetcode.\n' +
        'Filters and all possible variables are those assigned with\n' +
        '`set`, `unset` and shown with `show` commands. In particular\n' +
        'filters are: CATEGORY, LIMIT, SKIP and DIFFICULTY\n'
};
const FetchCommand = async (data, state) => {
    if (data.length < 1) {
        console.error(chalk_1.default.redBright("[ERROR] Fetch command uncorrectly formatted."));
        return state;
    }
    const spinner = new pprint_1.Spinner(""); // Create the spinner for dynamic logs
    // There are two possible ways to fetch a problem, the first one is by title-slug
    let question_id = Number.parseInt(data[2]) ?? null;
    if (!(data[2] !== undefined)) // Check if it is by id or by name 
     {
        // Check if the question has been already fetched
        const title_slug = data[1];
        spinner.changeMessage(`Checking if ${title_slug} is already fetched`);
        spinner.start();
        if (state.fetchedProblems) {
            const fetched = state.fetchedProblems.questions.filter((value) => value.titleSlug === title_slug);
            if (fetched.length > 0) {
                spinner.stop();
                console.log(chalk_1.default.greenBright("[INFO] Question already fetched"));
                return state;
            }
        }
        // If it does not exists, then we need to fetch the question id
        spinner.changeMessage(`Searching Question ID for ${title_slug}`);
        const problem_result = await lc.FetchQuestion({ titleSlug: title_slug });
        if (!problem_result || !problem_result?.question) {
            spinner.stop();
            console.error(chalk_1.default.redBright("[ERROR] No results found"));
            return state;
        }
        question_id = Number.parseInt(problem_result.question.questionId);
    }
    else {
        spinner.changeMessage(`Checking if Question ID ${question_id} is already fetched`);
        spinner.start();
        // The user has provided the question id, which is the question frontend id
        // most of the time, since it is the one that leetcode shows. There is no simple
        // way to discover the actual question id from the frontend one
        if (state.fetchedProblems) {
            const fetched = state.fetchedProblems.questions.filter((value) => Number.parseInt(value.questionFrontendId) === question_id);
            if (fetched.length > 0) {
                spinner.stop();
                console.log(chalk_1.default.greenBright("[INFO] Question already fetched"));
                return state;
            }
        }
    }
    spinner.changeMessage(`Fetching Problem ID: ${question_id}`);
    // We need to set the cookies for obtain the current status
    let header = formatter.FormatCookies(state.cookies);
    const problems_data = await lc.FetchProblemList({ categorySlug: '', limit: 1, skip: question_id - 1, filters: {} }, header);
    spinner.stop(); // Stops the spinner, that was the last operation
    if (!problems_data) {
        console.error(chalk_1.default.redBright(`[ERROR] Something went wrong or no problem with ID ${question_id}`));
        return state;
    }
    // Merge the fetched problem with other details from another query
    const question_title = problems_data.problemsetQuestionList.questions[0].titleSlug;
    const question_data = await lc.FetchQuestion({ titleSlug: question_title }, header);
    if (!question_data) {
        console.error(chalk_1.default.redBright(`[ERROR] Something went wrong.`));
        return state;
    }
    const detailed_data = generic.MergeStructures(question_data.question, problems_data.problemsetQuestionList.questions[0]);
    detailed_data.link = `https://leetcode.com/problems/${question_title}/description`;
    await (0, printer_1.PrintProblemsSummary)({ count: 1, questions: [detailed_data] }, state.variables);
    if (!state.fetchedProblems) {
        state.fetchedProblems = { count: 1, questions: [detailed_data] };
    }
    else {
        state.fetchedProblems.count++;
        state.fetchedProblems.questions.push(detailed_data);
    }
    return state;
};
// Fetch Command - "Download" a single question
exports.fetch_command = {
    group: 'Problems',
    name: 'Fetch Command',
    command: 'fetch',
    syntax: /^fetch\s+(title\s+([\w\-]+)|id\s+(\d+)){1,}$/,
    callback: FetchCommand,
    help: 'fetch <title|id> value - Fetch locally a single problem from remote. Notice\n' +
        'that: `title` is the title-slug which means the real title lower-case and spaces\n' +
        'replaced by `-`; `id` is the frontend id, i.e., the number identifying the problem.\n'
};
const DetailCommand = async (data, state) => {
    const problem_idx = Number.parseInt(data[0]); // Local index of the fetched problem
    // First we need to check for out of bound indexing
    if (!state.fetchedProblems) {
        console.error(chalk_1.default.yellowBright("[WARNING] Still no fetched problems. Please" +
            "fetch using either list or fetch commads."));
    }
    if (problem_idx >= state.fetchedProblems.count) {
        console.error(chalk_1.default.redBright(`[ERROR] Not enough fetched problems. Maximum ${state.fetchedProblems.count}`));
    }
    const problem_data = state.fetchedProblems.questions[problem_idx];
    console.log(problem_data);
    return state;
};
// Detail Command - Retrieve more informations about a problem
exports.detail_command = {
    group: 'Problems',
    name: 'Detail Command',
    command: 'detail',
    syntax: /^detail\s+(\d+)$/,
    callback: DetailCommand,
    help: 'detail idx - Print more details about a single problem.\n' +
        'The problem is specified using the `idx` parameter, which\n' +
        'identifies the local index of the question among those that\n' +
        'have already been fetched either using `list` or `fetch`.'
};
const CreateCommand = async (data, state) => {
    const spinner = new pprint_1.Spinner('');
    // Obtain the problem title from the current state if exists
    if (!state.fetchedProblems) {
        console.error(chalk_1.default.redBright("[ERROR] ") +
            chalk_1.default.redBright("No locally fetched problems. Consider running `fetch` or `list` command") +
            chalk_1.default.redBright("before `create`."));
        return state;
    }
    var problem_idxs;
    const fetch_length = state.fetchedProblems.count;
    if (data[0] !== undefined) {
        const problem_idx = Number.parseInt(data[0]);
        if (problem_idx >= fetch_length) {
            console.error(chalk_1.default.redBright("[ERROR] ") +
                chalk_1.default.redBright("Given index exceeds number of locally fetched problems. Consider\n") +
                chalk_1.default.redBright("increasing the `LIMIT` variable and run the `list` command again\n") +
                chalk_1.default.redBright("or directly locally fetching the specified problem either providing\n") +
                chalk_1.default.redBright("the frontend Id or the title-slug."));
            return state;
        }
        problem_idxs = [problem_idx];
    }
    else {
        problem_idxs = Array.from({ length: fetch_length }, (_, n) => n);
    }
    // Get already existing problem instances
    const existing_idxs = await generic.GetExistingProblems(state.variables["FOLDER"].value);
    for (let idx = 0; idx < problem_idxs.length; idx++) {
        const curr_problem = state.fetchedProblems.questions[problem_idxs[idx]];
        const frontend_id = Number.parseInt(curr_problem.questionFrontendId);
        if (existing_idxs.includes(frontend_id))
            continue;
        spinner.changeMessage(`Creating instance for ${curr_problem.title}`);
        spinner.start();
        generic.CreateQuestionInstance(curr_problem, state.variables["FOLDER"].value);
    }
    spinner.stop();
    return state;
};
// Create Command - "Download" a problem instance locally
exports.create_command = {
    group: 'Problems',
    name: 'Create Command',
    command: 'create',
    syntax: /^create(?:\s+(\d+))?$/,
    callback: CreateCommand,
    help: 'create <IDX> - Downloads a problem instance locally given the local index.\n' +
        'If the local index exists, it creates a new folder into the `FOLER` folder\n' +
        'named `<id>-<titleSlug>`. This folder will contain: a README.md file and a\n' +
        'index.html file with the problem description, and the source code file. Errors\n' +
        'when the input index is out of bound or if the list is empty. If no index is\n' +
        'specified, then it will downloads all the fetched problems.\n'
};
const DailyCommand = async (data, state) => {
    const spinner = new pprint_1.Spinner("Fetching daily question");
    spinner.start();
    // Check if the question has been already fetched
    if (state.dailyQuestion !== undefined) {
        console.log(chalk_1.default.greenBright('[INFO] Daily question already fetched.'));
    }
    else {
        const daily_question = await lc.FetchDailyQuestion(); // Fetch the daily question
        if (!daily_question) {
            spinner.stop();
            console.log(chalk_1.default.greenBright('[INFO] No daily question for today.'));
            return state;
        }
        // Once we have fetched the daily question we need to fetch the details
        const title_slug = daily_question.activeDailyCodingChallengeQuestion.question.titleSlug;
        const details = await lc.FetchQuestion({ titleSlug: title_slug });
        state.dailyQuestion = generic.MergeStructures(details.question, daily_question.activeDailyCodingChallengeQuestion.question);
        state.dailyQuestion.link = `https://leetcode.com${daily_question.activeDailyCodingChallengeQuestion.link}`;
        spinner.stop();
        (0, printer_1.PrintQuestionSummary)(state.dailyQuestion);
    }
    // Check if also the create command has been given
    if (data[0] !== undefined && data[0] === 'create') {
        spinner.changeMessage("Creating instance for daily question.");
        spinner.stop();
        const question_id = Number.parseInt(state.dailyQuestion.questionFrontendId);
        const folder = state.variables["FOLDER"].value;
        const existing_ids = await generic.GetExistingProblems(folder);
        if (existing_ids.includes(question_id)) {
            spinner.stop();
            console.log(chalk_1.default.greenBright('[INFO] Daily question instance already exists.'));
            return state;
        }
        // If it does not exists then create the instance
        generic.CreateQuestionInstance(state.dailyQuestion, folder);
    }
    spinner.stop();
    return state;
};
// Daily Command - Shows the question of today
exports.daily_command = {
    group: 'Problems',
    name: 'Daily Command',
    command: 'daily',
    syntax: /^daily(?:\s+(create))$/,
    callback: DailyCommand,
    help: 'daily [create] - Fetch the current daily question and optionally create an instance.'
};
