"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || (function () {
    var ownKeys = function(o) {
        ownKeys = Object.getOwnPropertyNames || function (o) {
            var ar = [];
            for (var k in o) if (Object.prototype.hasOwnProperty.call(o, k)) ar[ar.length] = k;
            return ar;
        };
        return ownKeys(o);
    };
    return function (mod) {
        if (mod && mod.__esModule) return mod;
        var result = {};
        if (mod != null) for (var k = ownKeys(mod), i = 0; i < k.length; i++) if (k[i] !== "default") __createBinding(result, mod, k[i]);
        __setModuleDefault(result, mod);
        return result;
    };
})();
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.MergeStructures = exports.ReadProblemTestCases = exports.ReadProblemSolution = exports.RequestPassword = exports.OpenLoginBrowser = exports.VerifyPassword = exports.HashPassword = exports.CreateQuestionInstance = exports.TimestampToDate = exports.Transpose = exports.ArraySum = exports.GetAllProblemsCount = exports.GetExistingProblems = void 0;
const promises_1 = require("fs/promises");
const path_1 = require("path");
const crypto_1 = require("crypto");
const formatter_1 = require("./formatter");
const constants_1 = __importDefault(require("../constants"));
const turndown_1 = __importDefault(require("turndown"));
const chalk_1 = __importDefault(require("chalk"));
const prompt_1 = __importDefault(require("prompt"));
const puppeteer_1 = __importDefault(require("puppeteer"));
const fs = __importStar(require("fs"));
const lc = __importStar(require("../leetcode"));
/**
 * Returns a list of all existing problems in the output instance folder
 */
const GetExistingProblems = async (problem_folder) => {
    try {
        var result = [];
        if (!fs.existsSync(problem_folder))
            return [];
        const files = await (0, promises_1.readdir)(problem_folder, { withFileTypes: true });
        result = files.map((value) => {
            return Number.parseInt(value.name.split("-")[0]);
        });
        return result;
    }
    catch (error) {
        console.error("Error reading folder: ", error);
        return [];
    }
};
exports.GetExistingProblems = GetExistingProblems;
/**
 * Returns for each difficulty filter the corresponding number of problems
 */
const GetAllProblemsCount = async () => ((await Promise.all(constants_1.default.DIFFICULTIES.map(async (x) => {
    const nproblems = await lc.FetchNumberOfProblems(x);
    return { [x]: nproblems ?? 0 };
}))).reduce((p, c) => ({ ...p, ...c })));
exports.GetAllProblemsCount = GetAllProblemsCount;
/**
 * Converts the input HTML content into Markdown and save it into the corresponding path
 */
const SaveHtmlToMarkdown = (path, content) => {
    // Initialize turndown service and convert
    const markdown = (new turndown_1.default()).turndown(content);
    fs.writeFileSync(path, markdown);
};
/**
 * Sum all values belonging to the array and returns the result
 */
const ArraySum = (...values) => values.reduce((x, y) => x + y);
exports.ArraySum = ArraySum;
/**
 * Transpose the input matrix
 */
const Transpose = (matrix) => {
    return matrix[0].map((_, colIndex) => matrix.map(row => row[colIndex]));
};
exports.Transpose = Transpose;
/**
 * Converts the input timestamp into datetime format
 */
const TimestampToDate = (timestamp) => ((new Date(timestamp * 1e3)).toLocaleString());
exports.TimestampToDate = TimestampToDate;
/**
 * Creates the question instance for the input problem data. In other words a new folder named
 * <question-id>-<titleSlug> if created into the `output` input path. This new directory will
 * contains five files named: README.md, index.html, solution.py, tests.txt and hints.txt
 */
const CreateQuestionInstance = (question, output) => {
    // Check if the input data is null
    if (question === null) {
        console.error(chalk_1.default.redBright("[ERROR] Impossible to create problem instance. Input is null."));
        return;
    }
    const question_root_folder = output || constants_1.default.OUTPATH;
    // Create also the root folder if it does not exists
    if (!fs.existsSync(question_root_folder)) {
        if (fs.mkdirSync(question_root_folder, { recursive: true }) === undefined) {
            console.error(chalk_1.default.redBright(`[ERROR] Impossible to create root folder: ${question_root_folder}`));
            return;
        }
    }
    // Create the problem folder name and file name
    const question_name = question.titleSlug;
    const question_id = question.questionFrontendId;
    const question_full_name = question_id.toString().concat("-", question_name);
    const question_folder = (0, path_1.join)(question_root_folder, question_full_name);
    const question_src = (0, path_1.join)(question_folder, "solution.py");
    const question_tests = (0, path_1.join)(question_folder, "tests.txt");
    const question_readme = (0, path_1.join)(question_folder, "README.md");
    const question_html = (0, path_1.join)(question_folder, "index.html");
    const question_tags = question.topicTags
        .map((value) => value.name)
        .reduce((prev, curr) => {
        return prev + ", " + curr;
    });
    // Python header with some information about the problem
    const python_header = "\"\"\"\n" +
        "QUESTION TITLE: " + question.title + "\n" +
        "QUESTION LINK: " + question.link + "\n" +
        "QUESTION TAGS: " + question_tags + "\n" +
        "QUESTION LEVEL: " + question.difficulty + "\n" +
        "\"\"\"";
    // Build the source code that will be written into the python file
    const source_code = "{0}\n\nfrom typing import *\n\n# SOLUTION STARTS\n{1}\n";
    // Take the correct code snippet (Python3)
    const code_snippet = question.codeSnippets
        .filter((value) => value.lang === 'Python3')
        .map((value) => value.code)[0] + "\n        ...";
    const full_source_code = (0, formatter_1.FormatString)(source_code, python_header, code_snippet);
    const tests = question.exampleTestcaseList
        .map((x) => x.replace('\n', ',,'))
        .join('\n');
    // First we need to create the question folder (it should not exist)
    fs.mkdir(question_folder, 0o777, (err) => {
        if (err)
            console.error(chalk_1.default.redBright(`[ERROR] Operation not permitted: ${err}`));
    });
    // Write the README file using the conversion operation
    SaveHtmlToMarkdown(question_readme, question.content);
    // Save the source code content into the source file
    fs.writeFileSync(question_src, full_source_code);
    fs.writeFileSync(question_html, question.content);
    fs.writeFileSync(question_tests, tests);
    // Log the result
    console.log(chalk_1.default.greenBright(`[INFO] Result written in folder: ${question_folder}`));
};
exports.CreateQuestionInstance = CreateQuestionInstance;
/**
 * Hash the input password using the salt. It uses Password-Based Key Derivation Function v2.
 */
const HashPassword = (password, salt) => {
    const iterations = constants_1.default.CRYPTO.ITERATIONS;
    const key_length = constants_1.default.CRYPTO.KEY_LENGTH;
    const digest = constants_1.default.CRYPTO.DIGEST;
    const hash = (0, crypto_1.pbkdf2Sync)(password, salt, iterations, key_length, digest);
    return hash.toString("hex");
};
exports.HashPassword = HashPassword;
/**
 * Checks that the input password and the input hash corresponds.
 */
const VerifyPassword = (password, salt, original_hash) => {
    return (0, exports.HashPassword)(password, salt) === original_hash;
};
exports.VerifyPassword = VerifyPassword;
/**
 * Open the login browser for the `login` command
 */
const OpenLoginBrowser = async (res_cb, req_cb) => {
    const browser = await puppeteer_1.default.launch({ headless: false, args: ['--window-size=1080,1024'] });
    const page = await browser.newPage();
    page.setRequestInterception(true);
    page.on('response', res_cb);
    page.on('request', req_cb);
    await page.goto(constants_1.default.SITES.LOGIN_PAGE.URL, { waitUntil: 'load' });
    page.setViewport({ width: 1080, height: 1024 });
    // Wait until the browser is not closed
    await new Promise((resolve) => {
        browser.on('disconnected', resolve);
    });
};
exports.OpenLoginBrowser = OpenLoginBrowser;
/**
 * Shows a prompt where the user can insert the password and verifying if the input is correct or not.
 * It provides `constants.CRYPTO.AUTH_ATTEMPTS` attempts.
 */
const RequestPassword = async (credentials) => {
    let attemps = constants_1.default.CRYPTO.AUTH_ATTEMPTS;
    prompt_1.default.message = '';
    prompt_1.default.delimiter = '';
    while (attemps > 0) {
        try {
            prompt_1.default.start({ noHandleSIGINT: true });
            const { password } = await prompt_1.default.get(constants_1.default.PROMPT.VALIDATION_SCHEMA);
            const validation_result = (0, exports.VerifyPassword)(password, credentials.salt, credentials.password);
            if (validation_result)
                return true;
        }
        catch (error) {
        }
        console.error(chalk_1.default.redBright("[ERROR] Wrong Password. Retry."));
        attemps--;
    }
    return false;
};
exports.RequestPassword = RequestPassword;
/**
 * Reads the solution from the solution.py file corresponding at the input question title
 */
const ReadProblemSolution = (folder, problem_id, problem_title) => {
    if (!fs.existsSync(folder)) {
        console.error(chalk_1.default.redBright(`[ERROR] ${folder} does not exists.`));
        return null;
    }
    try {
        const problem_file = (0, path_1.join)(folder, `${problem_id}-${problem_title}`, 'solution.py');
        const content = fs.readFileSync(problem_file, { encoding: 'utf-8' });
        // Split the string using the line terminator character
        const lines = content.split('\n');
        const final_lines = [];
        for (const line of lines) {
            if (line.includes('TESTS'))
                break;
            final_lines.push(line);
        }
        return final_lines.join('\n');
    }
    catch (error) {
        console.error(chalk_1.default.redBright("[ERROR]", error));
        return null;
    }
};
exports.ReadProblemSolution = ReadProblemSolution;
/**
 * Reads the test cases from the tests.txt file corresponding at the input question title
 */
const ReadProblemTestCases = (folder, problem_id, problem_title) => {
    if (!fs.existsSync(folder)) {
        console.error(chalk_1.default.redBright(`[ERROR] ${folder} does not exists.`));
        return null;
    }
    try {
        const test_file = (0, path_1.join)(folder, `${problem_id}-${problem_title}`, 'tests.txt');
        const content = fs.readFileSync(test_file, { encoding: 'utf-8' });
        const lines = content.split('\n');
        return lines.map((x) => x.replace(',,', '\n'));
    }
    catch (error) {
        console.error(chalk_1.default.redBright("[ERROR]", error));
        return null;
    }
};
exports.ReadProblemTestCases = ReadProblemTestCases;
/**
 * Merges multiple input objects to construct an object of return type
 */
const MergeStructures = (...itfs) => {
    return Object.assign({}, ...itfs);
};
exports.MergeStructures = MergeStructures;
