"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.PrintSubmissionResults = exports.PrintTestDetails = exports.PrintUserSummary = exports.PrintQuestionSummary = exports.PrintProblemsSummary = exports.PrintUsedFilters = void 0;
const pprint_1 = require("../pprint");
const formatter_1 = require("./formatter");
const general_1 = require("./general");
const chalk_1 = __importDefault(require("chalk"));
const constants_1 = __importDefault(require("../constants"));
const PrintUsedFilters = (vars) => {
    // Print the used filters
    const filter_table = new pprint_1.TablePrinter(undefined, undefined, [
        { size: 23, just: -1 },
        { size: 15, style: chalk_1.default.blueBright, just: -1 }
    ]);
    constants_1.default.APP.LIST_QUERY_VARIABLES.forEach((x) => {
        const var_type = vars[x].type; // Take the corresponding type
        const var_value = (var_type == "n") ? vars[x].value.toString() : vars[x].value;
        filter_table.pushRow(`Used Filter ${x}:`, var_value);
    });
    filter_table.showLine = false;
    console.log(filter_table.toString());
};
exports.PrintUsedFilters = PrintUsedFilters;
const PrintProblemsSummary = async (problems, vars) => {
    console.log("Fetched Problems :", problems.count);
    // Get the list of all problems already downloaded
    const downloaded_problems = await (0, general_1.GetExistingProblems)(vars['FOLDER'].value);
    // Create the table for the problem list
    const list_table = new pprint_1.TablePrinter(undefined, ['STATUS', 'IDX', 'ID', 'DIFFICULTY', 'TITLE', 'TAGS'], [
        { size: 8, just: 1 }, // Status column
        { size: 5, style: chalk_1.default.yellowBright }, // Local index column
        { size: 5, style: chalk_1.default.yellowBright }, // Problem ID column
        { size: 10, style: constants_1.default.APP.DIFFICULTY_STYLE }, // The difficulty column
        { size: 65, style: chalk_1.default.italic, just: -1 }, // The title column
        { size: 81, style: chalk_1.default.gray, just: -1 } // The tags column
    ]);
    problems.questions.forEach((value, idx) => {
        const tags = value.topicTags
            .map((value) => value.name)
            .reduce((p, c) => p + ", " + c);
        let status = '';
        // Check if the current question is already downloaded
        if (downloaded_problems.includes(Number.parseInt(value.questionFrontendId))) {
            status += constants_1.default.APP.EMOJIS.DOWNLOADED;
        }
        // Now check the status of the question
        if (value.status && value.status === "ac") {
            status += constants_1.default.APP.EMOJIS.CHECK;
        }
        else {
            status += constants_1.default.APP.EMOJIS.WRONG;
        }
        // Check if it is premium question
        if (value.paidOnly) {
            status += constants_1.default.APP.EMOJIS.NOT_FREE;
        }
        else {
            status += constants_1.default.APP.EMOJIS.FREE;
        }
        // Check if the solution is visible
        if (value.hasSolution && !value.solution.canSeeDetail) {
            status += constants_1.default.APP.EMOJIS.MONEY;
        }
        // Check if the video solution is available
        if (value.hasVideoSolution && !value.solution.paidOnlyVideo) {
            status += constants_1.default.APP.EMOJIS.HAS_VIDEO;
        }
        list_table.pushRow(status, idx, value.questionFrontendId, value.difficulty, value.title, tags);
    });
    list_table.showLine = false;
    console.log(list_table.toString());
};
exports.PrintProblemsSummary = PrintProblemsSummary;
const PrintQuestionSummary = (question) => {
    console.log((0, formatter_1.FormatString)("Last Question: { ID={0}, TitleSlug={1}, Link={2} }", question.questionFrontendId, question.titleSlug, question.link));
};
exports.PrintQuestionSummary = PrintQuestionSummary;
const PrintSubmissionStats = (submitStats) => {
    const total_subs = submitStats?.totalSubmissionNum;
    const acc_subs = submitStats?.acSubmissionNum;
    if (total_subs !== undefined && total_subs.length > 0) {
        const difficulties = total_subs.map((x) => x.difficulty);
        const columns = ['Difficulty', ...difficulties];
        const columns_props = Array(difficulties.length).fill({ size: 10, style: chalk_1.default.yellowBright });
        const props = [{ size: 12 }, ...columns_props];
        const tsubs_table = new pprint_1.TablePrinter("Total Submissions Count", columns, props);
        const counts = ['Count', ...total_subs.map((x) => x.count)];
        const subnum = ['Submissions', ...total_subs.map((x) => x.submissions)];
        const accepted = ['Accepted', ...acc_subs.map((x) => x.submissions)];
        tsubs_table.pushRow(...counts);
        tsubs_table.pushRow(...subnum);
        tsubs_table.pushRow(...accepted);
        console.log();
        console.log(tsubs_table.toString());
        console.log();
    }
};
const PrintShortSubmissionsDetails = (submissions) => {
    // For an optimal columns size, we should take the maximum between all sizes
    const sublist = submissions.submissionList;
    if (sublist.length < 1)
        return;
    const columns = constants_1.default.APP.SHORT_SUBMISSION.COLS;
    const colsizes = Array.from({ length: columns.length }, (_, i) => columns[i].length);
    sublist.forEach((x) => {
        const question_id_len = x.question?.questionId.length ?? 0;
        const lang_len = x.lang?.verboseName.length ?? 0;
        const runtime_len = x.runtimeDisplay?.length ?? 0;
        const memory_len = x.memoryDisplay?.length ?? 0;
        const timestamp = (0, general_1.TimestampToDate)(Number.parseInt(x.timestamp));
        colsizes[0] = Math.max(colsizes[0], x.id.length);
        colsizes[1] = Math.max(colsizes[1], x.title.length);
        colsizes[2] = Math.max(colsizes[2], question_id_len);
        colsizes[3] = Math.max(colsizes[3], timestamp.length);
        colsizes[4] = Math.max(colsizes[4], lang_len);
        colsizes[5] = Math.max(colsizes[5], runtime_len);
        colsizes[6] = Math.max(colsizes[6], memory_len);
    });
    const properties = colsizes.map((x, i) => ({
        size: x,
        style: constants_1.default.APP.SHORT_SUBMISSION.STYLES[i],
        just: constants_1.default.APP.SHORT_SUBMISSION.JUST[i]
    }));
    const table = new pprint_1.TablePrinter("Recent Accepted Submissions", columns, properties);
    table.showLine = false;
    sublist.forEach((x) => {
        const question_id = x.question?.questionId ?? "null";
        const lang = x.lang?.verboseName ?? "null";
        const runtime = x.runtimeDisplay ?? "null";
        const memory = x.memoryDisplay ?? "null";
        const timestamp = (0, general_1.TimestampToDate)(Number.parseInt(x.timestamp));
        table.pushRow(x.id, x.title, question_id, timestamp, lang, runtime, memory);
    });
    console.log();
    console.log(table.toString());
    console.log();
};
const PrintUserSummary = (user) => {
    console.log(`${chalk_1.default.bold("Username")}   : ${chalk_1.default.blueBright(user.profile?.username)}`);
    console.log(`${chalk_1.default.bold("Real Name")}  : ${chalk_1.default.gray(user.profile?.realName)}`);
    console.log(`${chalk_1.default.bold("Profile")}    : ${chalk_1.default.underline(user.link)}`);
    if (user.profile?.aboutMe !== undefined && user.profile?.aboutMe !== "") {
        const about_box = new pprint_1.RectangleBox(80, 0, chalk_1.default.italic);
        about_box.setTitle("About Me", 4);
        about_box.setContent(user.profile?.aboutMe, { upad: 1, dpad: 1, lpad: 2, rpad: 2 });
        console.log();
        console.log(about_box.toString());
        console.log();
    }
    console.log(`${chalk_1.default.bold("Reputation")} :`, user.profile?.reputation);
    console.log(`${chalk_1.default.bold("Ranking")}    :`, user.profile?.ranking);
    console.log(`${chalk_1.default.bold("Github")}     :`, chalk_1.default.underline(user.profile?.githubUrl));
    console.log(`${chalk_1.default.bold("Twitter")}    :`, chalk_1.default.underline(user.profile?.twitterUrl));
    console.log(`${chalk_1.default.bold("Linkedin")}   :`, chalk_1.default.underline(user.profile?.linkedinUrl));
    PrintSubmissionStats(user.submitStats);
    PrintShortSubmissionsDetails(user.acSubList);
};
exports.PrintUserSummary = PrintUserSummary;
const PrintTestDetails = (problem, result) => {
    console.log();
    console.log(`Test results for problem: ${chalk_1.default.bold(problem.title)}`);
    console.log();
    console.log(`${chalk_1.default.italic("Runtime")}     :`, result.status_runtime);
    console.log(`${chalk_1.default.italic("Memory")}      :`, result.status_memory);
    console.log(`${chalk_1.default.italic("Finish Time")} :`, (0, general_1.TimestampToDate)(result.task_finish_time));
    console.log(`${chalk_1.default.italic("Elased Time")} : ${result.elapsed_time / 1e3} [sec]`);
    console.log(`${chalk_1.default.italic("Language")}    :`, chalk_1.default.blueBright(result.pretty_lang));
    console.log();
    if (result.status_msg === "Runtime Error") {
        console.error(chalk_1.default.redBright("[ERROR] Runtime Error:", result.runtime_error));
        console.log();
        return;
    }
    const StatusDisplay = (x) => {
        if (x.includes('OK'))
            return chalk_1.default.greenBright(x);
        return chalk_1.default.redBright(x);
    };
    const table = new pprint_1.TablePrinter(undefined, ['OUTPUT', 'EXPECTED OUTPUT', 'TEST', 'STATUS'], [
        { size: 6, style: chalk_1.default.yellowBright },
        { size: 15, style: chalk_1.default.yellowBright },
        { size: 40, style: chalk_1.default.gray },
        { size: 6, style: StatusDisplay }
    ]);
    for (let i = 0; i < result.total_testcases; i++) {
        const output = result.code_answer[i];
        const expected = result.expected_code_answer[i];
        const test = result.test_cases[i].replace('\n', ', ');
        const status = (output === expected) ? 'OK' : 'NO';
        table.pushRow(output, expected, test, status);
    }
    console.log(table.toString());
    console.log();
};
exports.PrintTestDetails = PrintTestDetails;
const PrintSubmissionResults = (problem, submission) => {
    const failed = submission.total_correct < submission.total_testcases;
    const status = (failed) ? chalk_1.default.redBright("FAILED") : chalk_1.default.greenBright("SUCCESS");
    console.log();
    console.log(`Submission results for problem: ${chalk_1.default.bold(problem.title)}`);
    console.log();
    console.log(`${chalk_1.default.italic("Final Status")}  :`, status);
    console.log(`${chalk_1.default.italic("Submission Id")} :`, chalk_1.default.yellowBright(submission.submission_id));
    console.log(`${chalk_1.default.italic("Runtime")}       :`, submission.status_runtime);
    console.log(`${chalk_1.default.italic("Memory")}        :`, submission.status_memory);
    console.log(`${chalk_1.default.italic("Finish Time")}   :`, (0, general_1.TimestampToDate)(submission.task_finish_time));
    console.log(`${chalk_1.default.italic("Elased Time")}   : ${submission.elapsed_time / 1e3} [sec]`);
    console.log(`${chalk_1.default.italic("Language")}      :`, chalk_1.default.blueBright(submission.pretty_lang));
    console.log();
    if (submission.status_msg === "Runtime Error") {
        console.error(chalk_1.default.redBright("[ERROR] Runtime Error:", submission.runtime_error));
        console.log();
        return;
    }
    const table = new pprint_1.TablePrinter("Last Test Case", ['OUTPUT', 'EXPECTED', 'TEST CASE'], [
        { size: 20, style: chalk_1.default.yellowBright },
        { size: 20, style: chalk_1.default.yellowBright },
        { size: 40, style: chalk_1.default.gray }
    ]);
    table.showLine = false;
    table.pushRow(submission.code_output, submission.expected_output, submission.last_testcase);
    console.log(table.toString());
    console.log();
    const ShowsPerc = (x) => {
        if (x < 50)
            return chalk_1.default.redBright(x);
        if (x < 100)
            return chalk_1.default.rgb(255, 165, 0)(x);
        return chalk_1.default.greenBright(x);
    };
    const correct = submission.total_correct;
    const total = submission.total_testcases;
    const perc = correct / total * 100;
    console.log(`${chalk_1.default.italic("Result")} : ${correct}/${total} (${ShowsPerc(perc)} %)`);
    console.log();
};
exports.PrintSubmissionResults = PrintSubmissionResults;
